#[macro_use]
extern crate diesel;
extern crate csrf;
extern crate dotenv;
#[macro_use]
extern crate dotenv_codegen;

pub mod db_connection;
pub mod errors;
pub mod handlers;
pub mod models;
pub mod schema;

use actix_cors::Cors;
use actix_identity::{CookieIdentityPolicy, IdentityService};
use actix_web::{http::header, middleware::Logger, web, App, HttpServer};
use chrono::Duration;
use db_connection::get_pool;

#[actix_rt::main]
async fn main() -> std::io::Result<()> {
    std::env::set_var("RUST_LOG", "actix_web=debug,diesel=debug");
    env_logger::init();

    let bind = "localhost:8080";
    println!("Starting server at: http://{}", &bind);

    HttpServer::new(move || {
        App::new()
            .wrap(Logger::default())
            .wrap(
                Cors::new()
                    .allowed_origin(dotenv!("ALLOWED_ORIGIN"))
                    .allowed_methods(vec!["GET", "POST", "PUT", "DELETE"])
                    .allowed_headers(vec![
                        header::AUTHORIZATION,
                        header::CONTENT_TYPE,
                        header::ACCEPT,
                    ])
                    .max_age(3600)
                    .finish(),
            )
            .wrap(IdentityService::new(
                CookieIdentityPolicy::new(dotenv!("SECRET").as_bytes())
                    .domain(dotenv!("DOMAIN"))
                    .name("thesis")
                    .path("/")
                    .max_age(Duration::days(1).num_seconds())
                    .secure(dotenv!("COOKIE_SECURE").parse().unwrap()),
            ))
            .data(get_pool())
            .service(web::resource("/register").route(web::post().to(handlers::register::register)))
    })
    .bind(&bind)?
    .run()
    .await
}
